package com.miyzh.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.miyzh.dao.MedicineReturnDao;
import com.miyzh.entity.*;
import com.miyzh.entity.dto.MedicineReturnDto;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 出库信息同步
 */

@Slf4j
@Service
public class MedicineReturnService {

    @Autowired
    private MedicineReturnDao medicineReturnDao;
    @Value ("${postApi.returnUrl}")
    private String url;
    @Value("${postApi.pageSize}")
    private int pageSize;

    private class StockoutType {
        private static final String SELL = "1"; //销售
        private static final String LOSS = "2"; //损耗
        private static final String OUT_BOUND = "3"; //采购退货
        private static final String OTHER = "4"; // 其他
        private static final String ALLOT = "5"; // 调拨
    }

    private static final String MED_OUT = "1"; //西药销售出库记录
    private static final String CHI_MED_OUT = "2"; //中药销售出库记录
    private static final String MED_OUTTREASURY = "3"; //西药损耗/领用出库记录
    private static final String CHI_MED_OUTTREASURY = "4"; //中药损耗/领用出库记录
    private static final String MED_BACK = "5"; //西药采购退货出库记录
    private static final String CHI_MED_BACK = "6"; //中药采购退货出库记录
    private static final String MED_INV = "7"; //西药盘点盘亏出库记录
    private static final String CHI_MED_INV = "8"; //中药盘点盘亏出库记录

    @Async
    public void uploadMedicine(List<Clinic> clinicList, String flag) throws InterruptedException, JsonProcessingException {
        long startTime = System.currentTimeMillis();
        PageInfo<MedicineReturnTemp> page = new PageInfo<>();
        for(Clinic clinic : clinicList) {
            switch (flag) {
                case MED_OUT:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listMedicineOutbound(clinic.getClinicId());
                    });
                    break;
                case CHI_MED_OUT:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listChinaMedicineOutbound(clinic.getClinicId());
                    });
                    break;
                case MED_OUTTREASURY:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listMedicineOutTreasury(clinic.getClinicId());
                    });
                    break;
                case CHI_MED_OUTTREASURY:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listChinaMedicineOutTreasury(clinic.getClinicId());
                    });
                    break;
                case MED_BACK:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listMedicineBack(clinic.getClinicId());
                    });
                    break;
                case CHI_MED_BACK:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listChinaMedicineBack(clinic.getClinicId());
                    });
                    break;
                case MED_INV:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listMedicineInventory(clinic.getClinicId());
                    });
                    break;
                case CHI_MED_INV:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineReturnDao.listChinaMedicineInventory(clinic.getClinicId());
                    });
                    break;
            }
            if (page.getTotal() == 0) {
                continue;
            }
            Map<String, List<MedicineReturnTemp>> map =  page.getList()
                    .stream()
                    .collect(Collectors.groupingBy(MedicineReturnTemp::getStoreReturnCode));
            for(Map.Entry<String, List<MedicineReturnTemp>> msMap:map.entrySet()){
                MedicineReturnDto medicineReturnDto = new MedicineReturnDto();
                medicineReturnDto.setStoreCode(clinic.getStoreCode());
                switch (flag) {
                    case MED_OUT:
                    case CHI_MED_OUT:
                        // 销售出库
                        medicineReturnDto.setStockoutType(StockoutType.SELL);
                        break;
                    case MED_OUTTREASURY:
                    case CHI_MED_OUTTREASURY:
                        //
                        medicineReturnDto.setStockoutType(StockoutType.LOSS);
                        break;
                    case MED_BACK:
                    case CHI_MED_BACK:
                        medicineReturnDto.setStockoutType(StockoutType.OUT_BOUND);
                        break;
                    case MED_INV:
                    case CHI_MED_INV:
                        medicineReturnDto.setStockoutType(StockoutType.ALLOT);
                        break;
                }
                medicineReturnDto.setStorePrecordCode("00000000");
                medicineReturnDto.setReturnMan(msMap.getValue().get(0).getReturnMan());
                medicineReturnDto.setReturnTime(msMap.getValue().get(0).getReturnTime());
                medicineReturnDto.setStoreCode(clinic.getStoreCode());
                medicineReturnDto.setStoreReturnCode(msMap.getKey());
                List<MedicineReturn> list = msMap.getValue()
                        .stream()
                        .map(e -> {
                            MedicineReturn medicineReturn = new MedicineReturn();
                            BeanUtils.copyProperties(e, medicineReturn);
                            return medicineReturn;
                        })
                        .collect(Collectors.toList());
                medicineReturnDto.setMedicineInfos(list);
                this.upload(medicineReturnDto);
            }
            if (page.getPages() > 1) {
                PageInfo<MedicineReturnTemp> page1 = new PageInfo<>();
                for(int i = 2; i <= page.getPages(); i++) {
                    switch (flag) {
                        case MED_OUT:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listMedicineOutbound(clinic.getClinicId());
                            });
                            break;
                        case CHI_MED_OUT:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listChinaMedicineOutbound(clinic.getClinicId());
                            });
                            break;
                        case MED_OUTTREASURY:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listMedicineOutTreasury(clinic.getClinicId());
                            });
                            break;
                        case CHI_MED_OUTTREASURY:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listChinaMedicineOutTreasury(clinic.getClinicId());
                            });
                            break;
                        case MED_BACK:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listMedicineBack(clinic.getClinicId());
                            });
                            break;
                        case CHI_MED_BACK:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listChinaMedicineBack(clinic.getClinicId());
                            });
                            break;
                        case MED_INV:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listMedicineInventory(clinic.getClinicId());
                            });
                            break;
                        case CHI_MED_INV:
                            page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                medicineReturnDao.listChinaMedicineInventory(clinic.getClinicId());
                            });
                            break;
                    }
                    Map<String, List<MedicineReturnTemp>> map1 =  page1.getList()
                            .stream()
                            .collect(Collectors.groupingBy(MedicineReturnTemp::getStoreReturnCode));
                    for(Map.Entry<String, List<MedicineReturnTemp>> msMap:map1.entrySet()){
                        MedicineReturnDto medicineReturnDto = new MedicineReturnDto();
                        medicineReturnDto.setStoreCode(clinic.getStoreCode());
                        switch (flag) {
                            case MED_OUT:
                            case CHI_MED_OUT:
                                medicineReturnDto.setStockoutType(StockoutType.SELL);
                                break;
                            case MED_OUTTREASURY:
                            case CHI_MED_OUTTREASURY:
                                medicineReturnDto.setStockoutType(StockoutType.ALLOT);
                                break;
                            case MED_BACK:
                            case CHI_MED_BACK:
                                medicineReturnDto.setStockoutType(StockoutType.OUT_BOUND);
                                break;
                            case MED_INV:
                            case CHI_MED_INV:
                                medicineReturnDto.setStockoutType(StockoutType.LOSS);
                                break;
                        }
                        medicineReturnDto.setStorePrecordCode("00000000");
                        medicineReturnDto.setReturnMan(msMap.getValue().get(0).getReturnMan());
                        medicineReturnDto.setReturnTime(msMap.getValue().get(0).getReturnTime());
                        medicineReturnDto.setStoreCode(clinic.getStoreCode());
                        medicineReturnDto.setStoreReturnCode(msMap.getKey());
                        List<MedicineReturn> list = msMap.getValue()
                                .stream()
                                .map(e -> {
                                    MedicineReturn medicineReturn = new MedicineReturn();
                                    BeanUtils.copyProperties(e, medicineReturn);
                                    return medicineReturn;
                                })
                                .collect(Collectors.toList());
                        medicineReturnDto.setMedicineInfos(list);
                        this.upload(medicineReturnDto);
                    }
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("执行用时--{}", startTime - syncTime);
    };

    public void upload (Object o) throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();
        String params = objectMapper.writeValueAsString(o);
        System.out.println(params);
//        System.out.println(objectMapper.writeValueAsString(RestTemplateUtil.postHttp(url, params)));
    }

}
